
#include <stdio.h>

#include "serial.h"

//#define __TEST

#define MINFAKTOR (0.95)
#define MAXFAKTOR (1.05)

#define MINSTARTNULL  (773)     // 814 * 0.95
#define MAXSTARTNULL  (883)     // 814 * 1.05
#define MINSTARTEINS  (3146)    // 3312 * 0.95
#define MAXSTARTEINS  (3477)    // 3312 * 1.05

#define LIMIT         (1033)

typedef enum STATE {
  NONE,
  HALF  
} STATE;

#ifdef  __TEST

typedef struct RecData {
  short           samples;
  unsigned short  values[];
} RecData;


RecData Press1 = {{33},
  814, 3312, 813, 1253, 1469, 598, 814, 561, 
  813, 563, 814, 562, 815, 562, 778, 598, 
  815, 562, 814, 562, 815, 562, 813, 562, 
  814, 562, 815, 563, 814, 562, 778, 598, 
  814
};

RecData Repeat1 = {{32},
  815, 3310, 815, 562, 816, 1252, 815, 562, 
  815, 562, 815, 562, 777, 598, 813, 562, 
  815, 562, 815, 562, 814, 562, 1506, 561, 
  814, 562, 814, 1252, 1507, 1251, 814
};

RecData Press2 = {{33},
  816, 3311, 814, 1252, 1506, 562, 814, 562, 
  815, 562, 779, 598, 816, 561, 814, 562, 
  815, 561, 779, 598, 815, 562, 815, 562, 
  814, 562, 815, 561, 814, 561, 815, 562, 
  814
};

RecData Repeat2 = {{29},
  816, 3314, 813, 1253, 1506, 1252, 815, 562, 
  816, 561, 815, 561, 814, 562, 815, 562, 
  778, 598, 814, 562, 1505, 561, 814, 561, 
  814, 1252, 1506, 1251, 816
};

RecData Press3 = {{33},
  817, 3310, 815, 1251, 1542, 525, 816, 561, 
  816, 560, 817, 560, 815, 561, 815, 562, 
  816, 560, 815, 561, 815, 561, 815, 560, 
  815, 562, 817, 561, 815, 561, 815, 561, 
  815
};

RecData Repeat3 = {{32},
  817, 3313, 814, 562, 816, 561, 815, 1252, 
  815, 561, 815, 561, 815, 562, 814, 561, 
  852, 524, 814, 561, 816, 561, 1506, 560, 
  815, 562, 815, 1251, 1507, 1251, 816
};


#endif // __TEST

#if 1
unsigned char DBProt_CheckFor(short samples, unsigned short *data)
{
  if (samples > 16) {
    if (*data < MINSTARTNULL || *data > MAXSTARTNULL) {
      return 0;
    }
    data++;
    if (*data < MINSTARTEINS || *data > MAXSTARTEINS) {
      return 0;
    }
    data++;
    if (*data < MINSTARTNULL || *data > MAXSTARTNULL) {
      return 0;
    }
    return 1;
  }	
  return 0;
}
#endif


unsigned short  DBProt_Decode(short samples, unsigned short *data)
{
  unsigned short  result = 0;
  unsigned char   counter;
  STATE           status;
 
  data += 3;      // 010 ueberspringen, also gerade startnull ueberlesen  

  // Falls das 4. Bit lang ist, so wird die '1' als letzter Status uebenommen.
  status = (*data > LIMIT) ? HALF : NONE;
  data++;
  for (counter = 4; counter < samples; counter++) {   
    switch (status) {
    case HALF:
      result >>= 1;
      if (counter & 0x01) {
        result |= 0x8000;
      } 
      status = (*data > LIMIT) ? HALF : NONE;
      break;
    case NONE:
      status = HALF;
      break;
    }
    data++;
  }
  // Letztes Bit closen, falls das letzte Bit eine '1' ist..
  if (status == HALF && counter & 0x01) {
    result >>= 1;
    result |= 0x8000;
  }
	return result;
}


#ifdef  __TEST

int main (int argc, char **argv)
{
  if (DBProt_CheckFor(Press1.samples, Press1.values)) {
    DBProt_Decode(Press1.samples, Press1.values);
  }
  puts("");
  if (DBProt_CheckFor(Press2.samples, Press2.values)) {
    DBProt_Decode(Press2.samples, Press2.values);
  }
  puts("");
  if (DBProt_CheckFor(Press3.samples, Press3.values)) {
    DBProt_Decode(Press3.samples, Press3.values);
  }
  puts("");
  if (DBProt_CheckFor(Repeat1.samples, Repeat1.values)) {
    DBProt_Decode(Repeat1.samples, Repeat1.values);
  }
  puts("");
  if (DBProt_CheckFor(Repeat2.samples, Repeat2.values)) {
    DBProt_Decode(Repeat2.samples, Repeat2.values);
  }
  puts("");
  if (DBProt_CheckFor(Repeat3.samples, Repeat3.values)) {
    DBProt_Decode(Repeat3.samples, Repeat3.values);
  }
  puts("");
  return 0;
}

#endif // __TEST

