/******************************************************************************

  Messung von IR Pulsen - Puls/Pause Verhltnis
  MEGA8, 11.0592 MHz
  FB Traegerfrequenz = 36kHz, Periode = 28us

  Funktion:
  
  16 Bit Timer wird initalisiert & zur Zeitmessung verwendet.
  Ein Timerberlauf whrend der Eingang '1' ist fhrt zum Ende.
  Der Timer wird im externen Interrupt gesetzt / gelesen. 
  Wenn ein Ende erkannt wurde, so werden die Stnde ausgegeben.
  Ein Tick dauert ~723ns

  Durch die Buffersize wird die Anzahl Pulse pro Telegram hier auf
  63 begrenzt. 

******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr\pgmspace.h>
#include <avr/delay.h>

#include <stdlib.h>
#include <inttypes.h>


#include "irsample.h"
#include "usart.h"
#include "strio.h"
#include "toa.h"
#include "db.h"
#include "ps2.h"
#include "ps2proto.h"

typedef struct km {
	uint16_t	ircode;
	uint8_t		scancode;
} KeyMap PROGMEM;

static const KeyMap maps[] = {
	{0x5c00,	0x45, }, // 0
	{0x5c01,	0x16, }, // 1
	{0x5c02,	0x1e, }, // 2
	{0x5c03,	0x26, }, // 3
	{0x5c04,	0x25, }, // 4
	{0x5c05,	0x2e, }, // 5
	{0x5c06,	0x36, }, // 6
	{0x5c07,	0x3d, }, // 7
	{0x5c08,	0x3e, }, // 8
	{0x5c09,	0x46, }, // 9
	{0x5c0c,  0x1c, }, // a
	{0x5c0e,	0x33, }, // h
	{0x5c0f,	0x3b, }, // j
	{0x5c16,	0x3a, }, // m
	{0x5c17,	0x31, }, // n
	{0x5c20,	0x32, }, // b
	{0x5c27,	0x21, }, // c
	{0x5c28,	0x44, }, // o
	{0x5c2d,	0x23, }, // d
	{0x5c2e,	0x43, }, // i
	{0x5c2f,	0x42, }, // k
	{0x5c30,	0x4b, }, // l
	{0x5c3b,	0x34, }, // g
	{0x5c52,	0x2b, }, // f
	{0x5c55,	0x24, }, // e
	{0x5c82,	0x4d, }, // p
	{0x0000,	0x00, }, // end marker
};

static uint16_t RomU16(uint16_t addr) 
{
	uint16_t result;
	result = PRG_RDB(addr);
	result += ((uint16_t)PRG_RDB(++addr)) << 8 ;
	return result;
}

static char GetScanCode(uint16_t ir)
{
	int i = 0;
	
	uint16_t serachir = RomU16((uint16_t)&maps[i].ircode);
	while (serachir) {
		if (serachir == ir) {
			return PRG_RDB(&maps[i].scancode);
		}
		i++;
		serachir = RomU16((uint16_t)&maps[i].ircode);
	};
	return 0;
}

// Gibt Scancodes als kpl. Tastendruck aus.
static void PS2Output (uint8_t Command)
{
	PORTB &= ~_BV(PB1);
	LocalSendByte(Command);
	PS2_Proto();			
	_delay_loop_2(5000);    // 
	LocalSendByte(0xF0);    // Break Prefix senden
	PS2_Proto();			
	_delay_loop_2(5000);		// 
	LocalSendByte(Command);
	PS2_Proto();			
	_delay_loop_2(65000);		// Entprellung
	_delay_loop_2(65000);		// 
	_delay_loop_2(65000);		// 
	PORTB |= _BV(PB1);
}


/******************************************************************************

  Main

******************************************************************************/
int main(void)
{
	unsigned short val; 
	char tmp;

	initPS2IO();
  usart_init();
  ir_init();
	init_timer0();
 
	// LED1 bei Initalisierung aus. Diese Toggelt im IR Takt
	PORTB |= _BV(PB1);

  sei();                              // enable interrupts

  sputs ("DB2-IR V1.2 29.10.2003");

	_delay_loop_2(65000); // 
  LocalSendByte(0xaa);	
	_delay_loop_2(65000); // 

	while(1) {
		PS2_Proto();
    if (ir_ready) {
			if (DBProt_CheckFor(ir_index, ir_ticks)) {
				val = DBProt_Decode(ir_index, ir_ticks);
				tmp = GetScanCode(val);
				if (tmp) {
					PS2Output (tmp);
					// LED2 nach dem ersten empfangenen Telegramm ausmachen
					PORTB |= _BV(PB2);										
				} 
			}
      ir_init();        
    }
  }
}


